@@ -36,17 +36,14 @@ LearnerClassifRanger = R6Class("LearnerClassifRanger",
     #' Creates a new instance of this [R6][R6::R6Class] class.
     initialize = function() {
       ps = ps(
-        # FIXME: only works if predict_type == "se". How to set dependency?
-        alpha                        = p_dbl(default = 0.5, tags = "train"),
         always.split.variables       = p_uty(tags = "train"),
         class.weights                = p_uty(default = NULL, tags = "train"),
         holdout                      = p_lgl(default = FALSE, tags = "train"),
         importance                   = p_fct(c("none", "impurity", "impurity_corrected", "permutation"), tags = "train"),
         keep.inbag                   = p_lgl(default = FALSE, tags = "train"),
-        max.depth                    = p_int(default = NULL, lower = 0L, special_vals = list(NULL), tags = "train"),
+        max.depth                    = p_int(default = NULL, lower = 1L, special_vals = list(NULL), tags = "train"),
         min.bucket                   = p_int(1L, default = 1L, tags = "train"),
         min.node.size                = p_int(1L, default = NULL, special_vals = list(NULL), tags = "train"),
-        minprop                      = p_dbl(default = 0.1, tags = "train"),
         mtry                         = p_int(lower = 1L, special_vals = list(NULL), tags = "train"),
         mtry.ratio                   = p_dbl(lower = 0, upper = 1, tags = "train"),
         num.random.splits            = p_int(1L, default = 1L, tags = "train", depends = quote(splitrule == "extratrees")),
@@ -57,11 +54,10 @@ LearnerClassifRanger = R6Class("LearnerClassifRanger",
         regularization.factor        = p_uty(default = 1, tags = "train"),
         regularization.usedepth      = p_lgl(default = FALSE, tags = "train"),
         replace                      = p_lgl(default = TRUE, tags = "train"),
-        respect.unordered.factors    = p_fct(c("ignore", "order", "partition"), default = "ignore", tags = "train"),
+        respect.unordered.factors    = p_fct(c("ignore", "order", "partition"), tags = "train"),
         sample.fraction              = p_dbl(0L, 1L, tags = "train"),
         save.memory                  = p_lgl(default = FALSE, tags = "train"),
         scale.permutation.importance = p_lgl(default = FALSE, tags = "train", depends = quote(importance == "permutation")),
-        se.method                    = p_fct(c("jack", "infjack"), default = "infjack", tags = "predict"),
         seed                         = p_int(default = NULL, special_vals = list(NULL), tags = c("train", "predict")),
         split.select.weights         = p_uty(default = NULL, tags = "train"),
         splitrule                    = p_fct(c("gini", "extratrees", "hellinger"), default = "gini", tags = "train"),
