@@ -17,30 +17,40 @@ test_that("classif.ranger", {
     "dependent.variable.name", # handled via mlr3
     "status.variable.name", # handled via mlr3
     "classification", # handled via mlr3
-    "mtry.ratio" # custom hyperpar
+    "mtry.ratio", # custom hyperpar
+    "time.interest", # survival only
+    "alpha", # survival only
+    "minprop" # survival only
   )
 
-  ParamTest = run_paramtest(learner, fun, exclude)
+  ParamTest = run_paramtest(learner, fun, exclude, tag = "train")
   expect_true(ParamTest, info = paste0(
-    "Missing parameters:",
-    paste0("- '", ParamTest$missing, "'", collapse = ",")))
+    "\nMissing parameters in mlr3 param set:\n",
+    paste0("- ", ParamTest$missing, "\n", collapse = ""),
+    "\nOutdated param or param defined in additional control function not included in list of function definitions:\n",
+    paste0("- ", ParamTest$extra, "\n", collapse = ""))
+    )
 })
 
 test_that("predict classif.ranger", {
   learner = lrn("classif.ranger")
   fun = ranger:::predict.ranger
   exclude = c(
-    "quantiles", # required type not supported in mlr3
-    "what", # required type (quantiles) not supported in mlr3
+    "quantiles", # not supported by classification
+    "what", # not supported by classification
     "predict.all", # not supported in mlr3
     "formula", # handled via mlr3
     "object", # handled via mlr3
     "data", # handled via mlr3
-    "type" # handled via mlr3
+    "type", # handled via mlr3
+    "se.method" # regression only
   )
 
-  ParamTest = run_paramtest(learner, fun, exclude)
+  ParamTest = run_paramtest(learner, fun, exclude, tag = "predict")
   expect_true(ParamTest, info = paste0(
-    "Missing parameters:",
-    paste0("- '", ParamTest$missing, "'", collapse = ",")))
+    "\nMissing parameters in mlr3 param set:\n",
+    paste0("- ", ParamTest$missing, "\n", collapse = ""),
+    "\nOutdated param or param defined in additional control function not included in list of function definitions:\n",
+    paste0("- ", ParamTest$extra, "\n", collapse = ""))
+    )
 })
