@@ -66,3 +66,23 @@ test_that("default_values", {
   values = default_values(learner, search_space, task)
   expect_names(names(values), permutation.of =  c("replace", "sample.fraction", "num.trees", "mtry.ratio"))
 })
+
+test_that("quantile prediction", {
+  learner = mlr3::lrn("regr.ranger",
+    num.trees = 100,
+    predict_type = "quantiles",
+    quantiles = c(0.1, 0.5, 0.9),
+    quantile_response = 0.5)
+  task = tsk("mtcars")
+
+  learner$train(task)
+  pred = learner$predict(task)
+
+  expect_matrix(pred$quantiles, ncol = 3L)
+  expect_true(!any(apply(pred$quantiles, 1L, is.unsorted)))
+  expect_equal(pred$response, pred$quantiles[, 2L])
+
+  tab = as.data.table(pred)
+  expect_names(names(tab), identical.to = c("row_ids", "truth", "q0.1", "q0.5", "q0.9", "response"))
+  expect_equal(tab$response, tab$q0.5)
+})
